From 2bbd328cc35fdf2594587db1f13fcfcca7bee575 Mon Sep 17 00:00:00 2001
From: Amul Sul <amul.sul@enterprisedb.com>
Date: Tue, 14 Jan 2025 22:35:32 +0530
Subject: [PATCH v12 4/7] refactor: Change ATExecAlterConstrRecurse() argument.

Instead of passing the Relation of the referencing relation, we will
pass a relid. Opening the relation using the relid again should not
cause significant issues. However, this approach makes recursion more
efficient, especially in the later patch where we will be recreating
the referencing and referred triggers. In addition to that passing
relid of the referred relation too.

----
NOTE: This patch is not meant to be committed separately. It should
be squashed with the main patch that adds ENFORCED/NOT ENFORCED.
----
---
 src/backend/commands/tablecmds.c | 44 +++++++++++++++++---------------
 1 file changed, 23 insertions(+), 21 deletions(-)

diff --git a/src/backend/commands/tablecmds.c b/src/backend/commands/tablecmds.c
index bee18819290..c5447837fa9 100644
--- a/src/backend/commands/tablecmds.c
+++ b/src/backend/commands/tablecmds.c
@@ -392,14 +392,16 @@ static void AlterSeqNamespaces(Relation classRel, Relation rel,
 static ObjectAddress ATExecAlterConstraint(Relation rel, AlterConstraintStmt *cmdcon,
 										   bool recurse, bool recursing, LOCKMODE lockmode);
 static bool ATExecAlterConstrRecurse(AlterConstraintStmt *cmdcon, Relation conrel,
-									 Relation tgrel, Relation rel, HeapTuple contuple,
-									 List **otherrelids, LOCKMODE lockmode);
+									 Relation tgrel, const Oid fkrelid, const Oid pkrelid,
+									 HeapTuple contuple, List **otherrelids,
+									 LOCKMODE lockmode);
 static void AlterConstrTriggerDeferrability(Oid conoid, Relation tgrel, Relation rel,
 											bool deferrable, bool initdeferred,
 											List **otherrelids);
 static void ATExecAlterChildConstr(AlterConstraintStmt *cmdcon, Relation conrel,
-								   Relation tgrel, Relation rel, HeapTuple contuple,
-								   List **otherrelids, LOCKMODE lockmode);
+								   Relation tgrel, const Oid fkrelid, const Oid pkrelid,
+								   HeapTuple contuple, List **otherrelids,
+								   LOCKMODE lockmode);
 static ObjectAddress ATExecValidateConstraint(List **wqueue,
 											  Relation rel, char *constrName,
 											  bool recurse, bool recursing, LOCKMODE lockmode);
@@ -11906,8 +11908,9 @@ ATExecAlterConstraint(Relation rel, AlterConstraintStmt *cmdcon, bool recurse,
 		currcon->condeferred != cmdcon->initdeferred ||
 		rel->rd_rel->relkind == RELKIND_PARTITIONED_TABLE)
 	{
-		if (ATExecAlterConstrRecurse(cmdcon, conrel, tgrel, rel, contuple,
-									 &otherrelids, lockmode))
+		if (ATExecAlterConstrRecurse(cmdcon, conrel, tgrel, currcon->conrelid,
+									 currcon->confrelid, contuple, &otherrelids,
+									 lockmode))
 			ObjectAddressSet(address, ConstraintRelationId, currcon->oid);
 	}
 
@@ -11940,13 +11943,15 @@ ATExecAlterConstraint(Relation rel, AlterConstraintStmt *cmdcon, bool recurse,
  */
 static bool
 ATExecAlterConstrRecurse(AlterConstraintStmt *cmdcon, Relation conrel,
-						 Relation tgrel, Relation rel, HeapTuple contuple,
-						 List **otherrelids, LOCKMODE lockmode)
+						 Relation tgrel, const Oid fkrelid, const Oid pkrelid,
+						 HeapTuple contuple, List **otherrelids,
+						 LOCKMODE lockmode)
 {
 	Form_pg_constraint currcon;
 	Oid			conoid;
 	Oid			refrelid;
 	bool		changed = false;
+	Relation	rel;
 
 	/* since this function recurses, it could be driven to stack overflow */
 	check_stack_depth();
@@ -11955,6 +11960,8 @@ ATExecAlterConstrRecurse(AlterConstraintStmt *cmdcon, Relation conrel,
 	conoid = currcon->oid;
 	refrelid = currcon->confrelid;
 
+	rel = table_open(currcon->conrelid, lockmode);
+
 	/*
 	 * Update pg_constraint with the flags from cmdcon.
 	 *
@@ -12000,8 +12007,9 @@ ATExecAlterConstrRecurse(AlterConstraintStmt *cmdcon, Relation conrel,
 	 */
 	if (rel->rd_rel->relkind == RELKIND_PARTITIONED_TABLE ||
 		get_rel_relkind(refrelid) == RELKIND_PARTITIONED_TABLE)
-		ATExecAlterChildConstr(cmdcon, conrel, tgrel, rel, contuple,
-							   otherrelids, lockmode);
+		ATExecAlterChildConstr(cmdcon, conrel, tgrel, fkrelid, pkrelid,
+							   contuple, otherrelids, lockmode);
+	table_close(rel, NoLock);
 
 	return changed;
 }
@@ -12080,8 +12088,9 @@ AlterConstrTriggerDeferrability(Oid conoid, Relation tgrel, Relation rel,
  */
 static void
 ATExecAlterChildConstr(AlterConstraintStmt *cmdcon, Relation conrel,
-					   Relation tgrel, Relation rel, HeapTuple contuple,
-					   List **otherrelids, LOCKMODE lockmode)
+					   Relation tgrel, const Oid fkrelid, const Oid pkrelid,
+					   HeapTuple contuple, List **otherrelids,
+					   LOCKMODE lockmode)
 {
 	Form_pg_constraint currcon;
 	Oid			conoid;
@@ -12101,15 +12110,8 @@ ATExecAlterChildConstr(AlterConstraintStmt *cmdcon, Relation conrel,
 							   true, NULL, 1, &pkey);
 
 	while (HeapTupleIsValid(childtup = systable_getnext(pscan)))
-	{
-		Form_pg_constraint childcon = (Form_pg_constraint) GETSTRUCT(childtup);
-		Relation	childrel;
-
-		childrel = table_open(childcon->conrelid, lockmode);
-		ATExecAlterConstrRecurse(cmdcon, conrel, tgrel, childrel, childtup,
-								 otherrelids, lockmode);
-		table_close(childrel, NoLock);
-	}
+		ATExecAlterConstrRecurse(cmdcon, conrel, tgrel, fkrelid, pkrelid,
+								 childtup, otherrelids, lockmode);
 
 	systable_endscan(pscan);
 }
-- 
2.43.5

